/*
 * Lars
 * Temperedweaves clan and warzone system
 *
 * May 2001
 */

#include "conf.h"
#include "sysdep.h"
#include "string.h"

#include "structs.h"
#include "buffer.h"
#include "db.h"
#include "utils.h"
#include "spells.h"
#include "interpreter.h"
#include "comm.h"
#include "screen.h"
#include "clan.h"
#include "handler.h"
#include "diskio.h"

#define YES 	1
#define NO 	0

/*   external vars  */
extern struct clan_info *clan_index;
extern struct descriptor_data *descriptor_list;
extern char *class_abbrevs[];
extern FILE *player_fl;
extern int clan_top;
extern int mini_mud;

/* Local functions */
void build_clan_index(void);
void clan_save(int system, struct char_data *ch);
void write_clan(FBFILE *fl, int j);
int top_of_c_table = 0;
int top_of_c_file = 0;
long ctop_idnum = 0;
void init_clans(FBFILE *fl, int i);

/*
 * Clan levels
*/
const int CLAN_APPLY = 1;
const int CLAN_ENLISTED = 2;

const char *clan_names[] = {
  "&cYounglings&n",
  "&mDarkfriends&n",
  "&WWhite Tower&n",
  "&bSeanchan&n",
};

/* Define cross reference table */
int cross_reference[9999];

/* Define quick reference for colours (global) */
struct clan_colours clan_colour[8];

/* Define clan editing table */
struct clan_editing clan_edit[10];

/*
 * 			General functions go here
*/


/* generate index table for clan file */
void build_clan_index(void)
{
  int rec_count = 0, i;
  FBFILE *clan_index_file;
  FBFILE *fl;
  char index_name[40], line[256], buf[MAX_STRING_LENGTH], filename[40];
  char arg2[MAX_STRING_LENGTH];

  sprintf(index_name, "%s", CLAN_INDEX_FILE);
  if (!(clan_index_file = fbopen(index_name, FB_READ))) {
    top_of_c_table = -1;
    log("No clan index file.");
    return;
  }
  /* count the number of clans in the index */
  while (fbgetline(clan_index_file, line))
    if (*line != '~')
      rec_count++;
  fbrewind(clan_index_file);

  if (rec_count == 0) {
    clan_index = NULL;
    top_of_c_file = top_of_c_table = -1;
    return;
  }

  CREATE(clan_index, struct clan_info, rec_count);
  for (i = 0; i < rec_count; i++) {
    fbgetline(clan_index_file, line);
    sscanf(line, "%s", arg2);
    CREATE(clan_index[i].title, char, strlen(arg2) + 1);
    strcpy(clan_index[i].title, arg2);
    ctop_idnum = MAX(NUM_CLANS, rec_count);

    sprintf(filename, "%s/%s", CLAN_NEW_PREFIX, clan_index[i].title);
    log(filename);

    if (!(fl = fbopen(filename, FB_READ))) {
      sprintf(buf, "SYSERR: Couldn't open clan file %s", filename);
      log(buf);
      log("Clan initialization aborted.");
      return;
    }
    init_clans(fl, i);
  }

  sprintf(buf, "%d clans counted.", rec_count);
  log(buf);

  top_of_c_file = top_of_c_table = i - 1;

}

void save_clan_index(void)
{
  int i;
  FBFILE *index_file;
  
  if (!(index_file = fbopen(CLAN_INDEX_FILE, FB_WRITE))) {
    log("SYSERR:  Could not write clan index file");
    return;
  }

  for (i = 0; i <= top_of_c_table; i++)
    if (*clan_index[i].title)
      fbprintf(index_file, "%s\n", clan_index[i].title);
  fbprintf(index_file, "\n~");
  fbclose(index_file);
}

void save_clan(struct char_data * ch)
{
  FBFILE *fl;
  char outname[40], buf[MAX_STRING_LENGTH];
  int players_clannum = 0, ccmd = 0;

/*  if (ch == NULL) {
    sprintf(outname, "%s/%s", CLAN_NEW_PREFIX, CLANNAME(clan_index[players_clannum]));
  
    if (!(fl = fbopen(outname, FB_WRITE))) {
      sprintf(buf, "SYSERR: Couldn't open clan file %s for write", outname);
      mudlog(buf, NRM, LVL_GOD, TRUE);
      return;
    }
    log ("Saving all clans");
    for (i = 0; i < 4; i++) {
      write_clan(fl, clan_index[0].number);
      fbclose(fl);
    }
    return;
  } */

  if (!ch || ch == NULL) {
    log("NULL or missing ch in save_clan");
    return;
  }

  if (!((PLAYERCLANNUM(ch) == 0) && (CLANRANK(ch) == 0)))
    for (ccmd = 0; ccmd <= clan_top; ccmd++)
      if (CLANNUM(clan_index[ccmd]) == PLAYERCLAN(ch))
        players_clannum = ccmd;

  sprintf(outname, "%s/%s", CLAN_NEW_PREFIX, clan_index[PLAYERCLAN(ch)].title);

  if (!(fl = fbopen(outname, FB_WRITE))) {
    sprintf(buf, "SYSERR: Couldn't open clan file %s for write", outname);
    mudlog(buf, NRM, LVL_GOD, TRUE);
    return;
  }

  sprintf(buf, "Writing clan #%d, '%s'", PLAYERCLAN(ch), clan_names[PLAYERCLAN(ch)]);
  log(buf);

  write_clan(fl, PLAYERCLAN(ch));
  fbclose(fl);
  save_clan_index();

}

void init_clans(FBFILE *fl, int i)
{
  int num = 0;
  char buf[128], line[MAX_INPUT_LENGTH+1], tag[6];

    while(fbgetline(fl, line)) {
      tag_argument(line, tag);
      num = atoi(line);

      switch(*tag) {
      case 'C':
	if (!strcmp(tag, "Colr"))
	  clan_index[i].colour = num;

      case 'D':
	if (!strcmp(tag, "Desc"))
	  clan_index[i].description = fbgetstring(fl);

      case 'G':
	if (!strcmp(tag, "Gold"))
	  clan_index[i].clangold = num;

      case 'N':
	if (!strcmp(tag, "Numb"))
	  clan_index[i].number = num;
        if (!strcmp(tag, "Name"))
	  clan_index[i].title = str_dup(line);
        if (!strcmp(tag, "NumM"))
	  clan_index[i].nummembers = num;

      case 'R':
	if (!strcmp(tag, "Rank"))
	  clan_index[i].clantotrank = num;

      default:
	sprintf(buf, "SYSERR: Unknown tag %s in clanfile #%d", tag, i);
    }
  }
  fbclose(fl);

}

ACMD(do_clanlist)
{
  int i;

  send_to_char("Clan Listing\r\n\r\n", ch);

  for (i = 0; i <= top_of_c_table; i++) {
    sprintf(buf, "%18s - at war with :\r\n", clan_names[clan_index[i].number]);
    send_to_char(buf, ch);
  }

  sprintf(buf, "\r\n%d clans listed.\r\n", i);
  send_to_char(buf, ch);
}

ACMD(do_clanrank)
{
  int i;

  send_to_char("Clan Rankings\r\n\r\n", ch);

  for (i = 0; i <= top_of_c_table; i++) {
    sprintf(buf, "%18s : %d\r\n", clan_names[clan_index[i].number], clan_index[i].clantotrank);
    send_to_char(buf, ch);
  }

  sprintf(buf, "\r\n%d clans listed.\r\n", i);
  send_to_char(buf, ch);
}

ACMD(do_clansay)
{
  struct descriptor_data *pt;

  if (CLANRANK(ch) == 0) {
    send_to_char("But you don't belong to any clan!\r\n", ch);
    return;
  }

  if (!PRF_FLAGGED(ch, PRF_CLANTALK)) {
    send_to_char("Try turning your clan talk channel on first, dork!\r\n", ch);
    return;
  }    

  for (pt = descriptor_list; pt; pt = pt->next)
    if (!pt->connected && pt->character && 
        PLAYERCLAN(pt->character) == PLAYERCLAN(ch) && pt->character != ch && PRF_FLAGGED(pt->character, PRF_CLANTALK)) {
	  sprintf(buf, "%s clan-says, '%s'\r\n", GET_NAME(ch), argument);
	  send_to_char(buf, pt->character);
        }
    sprintf(buf, "You clan-say, '%s'\r\n", argument);
    send_to_char(buf, ch);
}

ACMD(do_enlist)
{
  struct char_data *victim;

  one_argument(argument, arg);

  if (!PLR_FLAGGED(ch, PLR_CLEADER))
    send_to_char("Huh?!?\r\n", ch);
  else if (CLANRANK(ch) <= 0)
    send_to_char("Huh?!?\r\n", ch);
  else if (!*arg)
    send_to_char("Enlist whom?\r\n", ch);
  else if (!(victim = get_player_vis(ch, arg, NULL, FIND_CHAR_ROOM)))
    send_to_char("You don't see them here.\r\n", ch);
  else if (CLANRANK(victim) > 0)
    send_to_char("They are already in a clan.\r\n", ch);
  else {
    CLANRANK(victim) = 1;
    PLAYERCLAN(victim) = PLAYERCLAN(ch);
    sprintf(buf, "You have been enlisted into the ranks of the '%s'.\r\n", clan_names[PLAYERCLAN(victim)]);
    send_to_char(buf, victim);
    send_to_char("You have enlisted them into your ranks.\r\n", ch);
    sprintf(buf, "%s has been enlisted into the ranks of the '%s&y'", GET_NAME(victim), clan_names[PLAYERCLAN(victim)]);
    infolog(buf);
    SET_BIT(PRF_FLAGS(victim), PRF_CLANTALK);
  }
}

ACMD(do_assign)
{
  struct char_data *victim;
  int num;

  two_arguments(argument, buf, buf2);

  if (!*buf)
    send_to_char("Whom do you wish to assign to a clan?\r\n", ch);
  else if (!(victim = get_char_vis(ch, buf, NULL, FIND_CHAR_WORLD)))
    send_to_char(NOPERSON, ch);
  else if (victim == ch)
    send_to_char("You can't assign yourself.\r\n", ch);
  else if (!*buf2 || (num = atoi(buf2)) >= NUM_CLANS) {
    send_to_char("To which clan would you like them assigned?\r\n", ch);
    send_to_char("The available choices are:\r\n", ch);
    send_to_char(" 0 - Younglings\r\n", ch);
    send_to_char(" 1 - Darkfriends\r\n", ch);
    send_to_char(" 2 - White Tower\r\n", ch);
    send_to_char(" 3 - Seanchan\r\n", ch);
  } else if (CLANRANK(victim) > 0)
    send_to_char("They are already enlisted in a clan.\r\n", ch);
  else {
    CLANRANK(victim) = 1;
    PLAYERCLAN(victim) = num;
    PLAYERCLANNUM(victim) = num;
    SET_BIT(PRF_FLAGS(victim), PRF_CLANTALK);
    send_to_char("They have been granted leadership of the clan.\r\n", ch);
    sprintf(buf, "You have been given control of the '%s'.\r\n", clan_names[clan_index[num].number]);
    send_to_char(buf, victim);
    sprintf(buf, "%s has been granted leadership of the '%s&y'", GET_NAME(victim), clan_names[clan_index[num].number]);
    infolog(buf);
    SET_BIT(PLR_FLAGS(victim), PLR_CLEADER);
  }
}

ACMD(do_resign)
{
  if (CLANRANK(ch) <= 0)
    send_to_char("Huh?!?\r\n", ch);
  else {
    if (PLR_FLAGGED(ch, PLR_CLEADER))
       REMOVE_BIT(PLR_FLAGS(ch), PLR_CLEADER);
    send_to_char("You have resigned from your clan.\r\n", ch);
    sprintf(buf, "%s has resigned from the '%s&y'", GET_NAME(ch), clan_names[PLAYERCLAN(ch)]);
    infolog(buf);
    CLANRANK(ch) = 0;
    PLAYERCLAN(ch) = 0;
  }
}
    
void write_clan(FBFILE *fl, int j)
{
  log("Write clan");

  fprintf(stderr, "Writing clan #%d which is '%s'\n", j, clan_index[j].title);

  fbprintf(fl, "Numb: %d\n", clan_index[j].number);
  fbprintf(fl, "Name: %s\n", clan_index[j].title);
  fbprintf(fl, "Colr: %d\n", clan_index[j].colour);
  fbprintf(fl, "Gold: %d\n", clan_index[j].clangold);
  fbprintf(fl, "Rank: %d\n", clan_index[j].clantotrank);
  fbprintf(fl, "NumM: %d\n", clan_index[j].nummembers);
}

ACMD(do_clanwho)
{
  struct descriptor_data *d;
  struct char_data *tch;
  const char *castle1owner = NULL, *castle2owner = NULL;

  for (d = descriptor_list; d; d = d->next) {
    if ((STATE(d) != CON_PLAYING) && ((STATE(d) <= CON_DISCONNECT)))
      continue;

    if (d->original)
      tch = d->original;
    else if (!(tch = d->character))
      continue;

    if (CLANRANK(tch) > 0) {
      sprintf(buf2, "[%s]", clan_names[PLAYERCLAN(tch)]);
      sprintf(buf, "%18s - %s [&y0&n/&y0&n]\r\n", buf2, GET_NAME(tch));
      send_to_char(buf, ch);
      if (clan_index[PLAYERCLAN(tch)].own_castle == 1)
        castle1owner = clan_names[clan_index[PLAYERCLAN(tch)].number];
      if (clan_index[PLAYERCLAN(tch)].own_castle == 2)
	castle2owner = clan_names[clan_index[PLAYERCLAN(tch)].number];
    }
  }

  send_to_char("\r\n", ch);

  if (castle1owner == NULL)
    sprintf(buf, "Owner of Castle One:  No One\r\n");
  else
    sprintf(buf, "Owner of Castle One:  %s\r\n", castle1owner);

  if (castle2owner == NULL)
    sprintf(buf + strlen(buf), "Owner of Castle Two:  No One\r\n");
  else
    sprintf(buf + strlen(buf), "Owner of Castle Two:  %s\r\n", castle2owner);

  send_to_char(buf, ch);
}

